/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/**
 *
 */
package com.sos.VirtualFileSystem.Filter.Options;

import java.util.Vector;

import org.apache.log4j.Logger;

import com.sos.VirtualFileSystem.Filter.SOSBase64DecodeFilter;
import com.sos.VirtualFileSystem.Filter.SOSBase64EncodeFilter;
import com.sos.VirtualFileSystem.Filter.SOSDos2UnixFilter;
import com.sos.VirtualFileSystem.Filter.SOSExcludeIncludeRecordsFilter;
import com.sos.VirtualFileSystem.Filter.SOSNullFilter;
import com.sos.VirtualFileSystem.Filter.SOSRecordsFilter;
import com.sos.VirtualFileSystem.Filter.SOSSearchAndReplaceFilter;
import com.sos.VirtualFileSystem.Filter.SOSUnix2DosFilter;
import com.sos.VirtualFileSystem.Filter.Options.SOSOptionFilterSequence.enuFilterCodes;
import com.sos.VirtualFileSystem.Interfaces.ISOSVirtualFile;

/**
 * @author KB
 *
 */
public class SOSFilterOptions extends SOSFilterOptionsSuperClass {

	private final String			conClassName		= this.getClass().getSimpleName();
	@SuppressWarnings("unused")
	private static final String		conSVNVersion		= "$Id: SOSFilterOptions.java 24080 2014-05-02 11:50:10Z kb $";
	@SuppressWarnings("unused")
	private final Logger			logger				= Logger.getLogger(this.getClass());

	private Vector<SOSNullFilter>	lstFilters			= null;

	/**
	 *
	 */
	private static final long		serialVersionUID	= 555337165999031797L;

	public Vector<SOSNullFilter> getFilter() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getFilter";

		if (lstFilters == null) {
			lstFilters = new Vector();
			if (FilterSequence.isDirty() == true) {
				for (String strFilterName : FilterSequence.getValueList()) {
					lstFilters.add(getFilterInstance(strFilterName));
				}
			}
			else {
				if (exclude_lines_after.isDirty() || excludeLinesBefore.isDirty()) { // name: records
					SOSNullFilter objF = new SOSRecordsFilter(this);
					lstFilters.add(objF);
				}
				if (excludeEmptyLines.isDirty() || excludeLines.isDirty() || includeLines.isDirty()) { // name: excludeinclude
					SOSNullFilter objF = new SOSExcludeIncludeRecordsFilter(this);
					lstFilters.add(objF);
				}
			}
		}
		return lstFilters;

	} // private Vector <SOSNullFilter> getFilter

	public void startPostProcessing(final ISOSVirtualFile objOutput) {
		for (SOSNullFilter sosNullFilter : lstFilters) {
			byte[] strB = sosNullFilter.getBuffer();
			if (strB != null && objOutput != null) {
				objOutput.write(strB);
			}
		}
	}

	public void startCloseProcessing(final ISOSVirtualFile objOutput) {
		for (SOSNullFilter sosNullFilter : lstFilters) {
			sosNullFilter.close();
		}
	}

	// TODO in die Klasse SOSFilterOption verschieben.
	public SOSNullFilter getFilterInstance(final String pstrFilterName) {
		SOSNullFilter objF = null;
		for (enuFilterCodes enuFC : enuFilterCodes.values()) {
			if (enuFC.name().equalsIgnoreCase(pstrFilterName)) {
				switch (enuFC) {
					// TODO eine Klasse, die die Filter Instanz liefert und auch den Filter-Type liefert: stream, endpoint, startpoint, ... Ebenso fr Validator einsetzen
					case dos2unix: //
						objF = new SOSDos2UnixFilter(this);
						break;
					case unix2dos: //
						objF = new SOSUnix2DosFilter(this);
						break;
					case searchreplace: //
						objF = new SOSSearchAndReplaceFilter(this);
						break;
					case excludeinclude: //
						objF = new SOSExcludeIncludeRecordsFilter(this);
						break;
					case base64encode: //
						objF = new SOSBase64EncodeFilter(this);
						break;
					case base64decode: //
						objF = new SOSBase64DecodeFilter(this);
						break;
					case md5filter: //
						//				objF = new SOSMd5Filter(this);
						break;
					case nullfilter:
						objF = new SOSNullFilter(this);
						break;

					case records:
						objF = new SOSRecordsFilter(this);
						break;

					default:
						break;
				}
				break;  // leave for
			}
		}
		return objF;
	}
}
