/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/**
 * 
 */
package com.sos.VirtualFileSystem.Filter.Options;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Options.JSOptionsClass;
import com.sos.JSHelper.Options.SOSOptionStringValueList;

/**
 * @author KB
 *
 */
public class SOSOptionFilterSequence extends SOSOptionStringValueList {

	public enum enuFilterCodes {
		dos2unix, //
		unix2dos, //
		searchreplace, //
		excludeinclude, //
		base64encode, //
		base64decode, //
		md5filter, //
		records, // 
		nullfilter
	};
	private final enuFilterCodes	objFC				= null;

	/**
	 * 
	 */
	private static final long		serialVersionUID	= -3306255818817394338L;

	public SOSOptionFilterSequence(final String pstrValue) {
		super(null, "nullFilter", "description", "", false);
		this.Value(pstrValue);
	}

	public SOSOptionFilterSequence(final JSOptionsClass pPobjParent, final String pPstrKey, final String pPstrDescription, final String pPstrValue,
			final String pPstrDefaultValue, final boolean pPflgIsMandatory) {
		super(pPobjParent, pPstrKey, pPstrDescription, pPstrValue, pPstrDefaultValue, pPflgIsMandatory);

	}

	public SOSOptionFilterSequence(final JSOptionsClass pobjParent, final String pstrIndexedKey, final String pstrDescription, final String pstrDefaultValue,
			final boolean pflgIsMandatory) {
		super(pobjParent, pstrIndexedKey, pstrDescription, pstrDefaultValue, pflgIsMandatory);
	}

	@Override
	public void Value(final String pstrValueList) {
		if (isNotEmpty(pstrValueList)) {
			super.Value(pstrValueList);
			for (String strSingleValue : strValueList) {
				boolean flgFound = false;
				for (enuFilterCodes enuFC : enuFilterCodes.values()) {
					if (enuFC.name().equalsIgnoreCase(strSingleValue)) {
						flgFound = true;
						break;
					}
				}

				if (flgFound == false) {
					String strM = String.format("Invalid Filter name found '%1$s'.", strSingleValue);
					String strT = "\nValid Filter names are: ";
					for (enuFilterCodes enuFC : enuFilterCodes.values()) {
						strT = strT + enuFC.name() + ",";
					}
					strT = strT.substring(1, strT.length() - 1);
					throw new JobSchedulerException(strM + strT);
				}
			}
		}
		else {
			throw new JobSchedulerException("An empty Filter list is not supported");
		}
	}
}
