/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.eventing.dialog.classes;

import java.util.prefs.Preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

import com.sos.dashboard.globals.SOSDashboardOptions;

public class SOSTabEVENTS extends CTabItem {

	private static final String TABNAME_SCHEDULER_EVENTLIST = "Events";
	private static final String TABNAME_SCHEDULER_EVENTHANDLERLIST = "Event Handler";

	private final Composite composite;
	private Preferences prefs;
	private CTabFolder mainTabFolder = null;
	private String host;
	private int port;
	private String eventhandlerDirectory;
	private SOSTabEventHandlerList tbtmEventHandlerList;
	private SOSTabEventList tbtmEventList;
    private SOSDashboardOptions           objOptions                          = null;

	public SOSTabEVENTS(final SOSDashboardOptions objOptions_,final String caption, final CTabFolder parent ) {
		super(parent, SWT.NONE);
		objOptions = objOptions_;
		setText(caption);
		composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout(1, false);
		composite.setLayout(layout);
		createContents();
		composite.layout();
		this.setControl(composite);

	}

	public void disableRefresh(){
		 if (tbtmEventList != null){
 			 tbtmEventList.disableRefresh();
 			 tbtmEventHandlerList.disableRefresh();
		 }

	}

	public void enableRefresh(){
		 if (tbtmEventList != null){
			 tbtmEventList.enableRefresh();
			 tbtmEventHandlerList.enableRefresh();
		 }

	}
	/**
	 * Create contents of the window
	 */
	protected void createContents() {
		prefs = Preferences.userNodeForPackage(this.getClass());

		host = prefs.node("EVENT_CONFIGURATION").get("HOST", "localhost");
		port = Integer.valueOf(prefs.node("EVENT_CONFIGURATION").get("PORT", "4444"));
		eventhandlerDirectory = prefs.node("EVENT_CONFIGURATION").get("EVENTHANDLERDIRECTORY", objOptions.Scheduler_Data.Value() + "/config/events");

		mainTabFolder = new CTabFolder(composite, SWT.NONE);
        mainTabFolder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		mainTabFolder.setSelectionBackground(Display.getCurrent().getSystemColor(SWT.COLOR_TITLE_INACTIVE_BACKGROUND_GRADIENT));

		tbtmEventList = new SOSTabEventList(TABNAME_SCHEDULER_EVENTLIST, mainTabFolder, host, port);
		tbtmEventHandlerList = new SOSTabEventHandlerList(TABNAME_SCHEDULER_EVENTHANDLERLIST, mainTabFolder, host, port, eventhandlerDirectory);
		tbtmEventHandlerList.setObjOptions(objOptions);

		mainTabFolder.setSelection(0);
		createMenue(mainTabFolder);
		createMenue(tbtmEventList.getParentComposite());
		createMenue(tbtmEventList.getTable());
		createMenue(tbtmEventHandlerList.getParentComposite());
		createMenue(tbtmEventHandlerList.getTable());
	}

	private void createMenue(final Composite parent) {

		if (parent != null) {
			Menu contentMenu = new Menu(parent);
			parent.setMenu(contentMenu);
			MenuItem addItem = new MenuItem(contentMenu, SWT.PUSH);
			addItem.setText("Configuration");
			addItem.addSelectionListener(new org.eclipse.swt.events.SelectionListener() {
				@Override
				public void widgetSelected(final org.eclipse.swt.events.SelectionEvent e) {
					SosDialogEventConfiguration s = new SosDialogEventConfiguration(composite.getShell(), host, port, eventhandlerDirectory);

					if (s.getHost() != null) {
						prefs.node("EVENT_CONFIGURATION").put("HOST", s.getHost());
						host = s.getHost();
	                    tbtmEventHandlerList.setHost(host);
	                    tbtmEventList.setHost(host);
					}

					prefs.node("EVENT_CONFIGURATION").put("PORT", s.getPort());
					port = Integer.valueOf(s.getPort());
					tbtmEventHandlerList.setPort(port);
					tbtmEventList.setPort(port);

					if (s.getEventhandlerDirectory() != null) {
						prefs.node("EVENT_CONFIGURATION").put("EVENTHANDLERDIRECTORY", s.getEventhandlerDirectory());
						eventhandlerDirectory = s.getEventhandlerDirectory();
						tbtmEventHandlerList.setEventhandlerDirectory(eventhandlerDirectory);
					}
				}

				@Override
				public void widgetDefaultSelected(final org.eclipse.swt.events.SelectionEvent e) {
				}
			});
		}
	}



}
