/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.graphviz;

/**
 * Class to convert the hex representation of a color to the red, green and blue part and vice versa.
 *
 * @uthor Stefan Schaedlich
 * at 25.09.13 15:44
 */
public class RGBColor {

    private final Integer red;
    private final Integer green;
    private final Integer blue;
    private final String hexString;

    public RGBColor(String hexString) {
        Integer bigint = Integer.parseInt(hexString.substring(1), 16);
        this.red = (bigint >> 16) & 255;
        this.green = (bigint >> 8) & 255;
        this.blue = bigint & 255;
        this.hexString = hexString;
    }

    public RGBColor(Integer red, Integer green, Integer blue) {
        this.hexString = "#" + componentToHex(red) + componentToHex(green) + componentToHex(blue);
        this.red = red;
        this.green = green;
        this.blue = blue;
    }

    private String componentToHex(Integer c) {
        String hex = c.toString(16);
        return (hex.length() == 1) ? "0" + hex : hex;
    }

    public int getRed() {
        return red;
    }

    public int getGreen() {
        return green;
    }

    public int getBlue() {
        return blue;
    }

    public String getHexString() {
        return hexString;
    }

}
