/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.graphviz;

import com.sos.graphviz.enums.RankType;

public class Subgraph extends Graph {

	private final SubgraphProperties subgraphProperties;
    private final Graph parent;

	protected Subgraph(String subgraphId, RankType rankType, Graph parent) {
		super(subgraphId);
		subgraphProperties = new SubgraphProperties(rankType);
        this.parent = parent;
		setGlobalNodeProperties(parent.getGlobalNodeProperties());
    }
	
	@Override
	public String getContent() {
		StringBuilder sb = new StringBuilder();
		sb.append( subgraphProperties.getContent() );
		sb.append(getGlobalNodeProperties().getSource());
		sb.append( getMainContent() );
		return sb.toString();
	}

	public GraphvizObject getProperties() {
		return subgraphProperties;
	}

	/**
	 * the graph properties should be available via the Graph object only.
	 * 
	 * @see com.sos.graphviz.Graph#getGraphProperties()
	 */
	public GraphProperties getGraphProperties() {
		return null;
	}

	public SubgraphProperties getSubgraphProperties() {
		return subgraphProperties;
	}

    public Graph getParent() {
        return parent;
    }

    @Override
    /**
     * Edges are defined always at level of RootGraph
     */
    public Edge newEdge(Node nodeFrom, Node nodeTo) {
        return getRootGraph().newEdge(nodeFrom,nodeTo);
    }

    private Graph getRootGraph() {
        Graph g = this.getParent();
        while (g instanceof Subgraph) {
            g = ((Subgraph) g).getParent();
        }
        return g;
    }

}

