/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/


package com.sos.jitl.eventing.checkevents;

import java.io.File;

import com.sos.jitl.eventing.checkevents.JobSchedulerCheckEvents;
import com.sos.jitl.eventing.checkevents.JobSchedulerCheckEventsOptions;
import sos.spooler.Spooler;
import sos.scheduler.job.JobSchedulerJobAdapter;  // Super-Class for JobScheduler Java-API-Jobs
import org.apache.log4j.Logger;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.localization.*;
/**
 * \class 		JobSchedulerCheckEventsJSAdapterClass - JobScheduler Adapter for "Check if events exist"
 *
 * \brief AdapterClass of JobSchedulerCheckEvents for the SOSJobScheduler
 *
 * This Class JobSchedulerCheckEventsJSAdapterClass works as an adapter-class between the SOS
 * JobScheduler and the worker-class JobSchedulerCheckEvents.
 *

 *
 *
 * \verbatim ;
 * \endverbatim
 */
public class JobSchedulerCheckEventsJSAdapterClass extends JobSchedulerJobAdapter  {
	private final String					conClassName						= "JobSchedulerCheckEventsJSAdapterClass";
	private static Logger		logger			= Logger.getLogger(JobSchedulerCheckEventsJSAdapterClass.class);
	private boolean success=false;

	public void init() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::init";
		doInitialize();
	}

	private void doInitialize() {
	} // doInitialize

	@Override
	public boolean spooler_init() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::spooler_init";
		return super.spooler_init();
	}

	@Override
	public boolean spooler_process() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::spooler_process";

		try {
			super.spooler_process();
			doProcessing();
		}
		catch (Exception e) {
            throw new JobSchedulerException("Fatal Error:" + e.getMessage(), e);
   		}
		finally {
		} // finally
		
       if (success) {
           return signalSuccess();
       }else {
           return false;
           //return signalFailure();
       }

	} // spooler_process

	@Override
	public void spooler_exit() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::spooler_exit";
		super.spooler_exit();
	}

	private void doProcessing() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::doProcessing";

        Object objSp = getSpoolerObject();

		JobSchedulerCheckEvents objR = new JobSchedulerCheckEvents();
		JobSchedulerCheckEventsOptions objO = objR.Options();

		
        String configuration_file = "";
        Spooler objSpooler = (Spooler) objSp;

        if (objO.configuration_file.IsEmpty() == false) {
            logger.debug("configuration_file from param");
            configuration_file = objO.configuration_file.Value();
        }else {
            logger.debug("configuration_file from scheduler");
            File f = new File(new File(objSpooler.configuration_directory()).getParent(),"hibernate.cfg.xml");
            if (!f.exists()){
               f = new File(new File(objSpooler.directory()),"config/hibernate.cfg.xml");
            }
            configuration_file = f.getAbsolutePath();
        }
        objO.configuration_file.Value(configuration_file);


		
        objO.CurrentNodeName(this.getCurrentNodeName());
		objO.setAllOptions(getSchedulerParameterAsProperties(getJobOrOrderParameters()));
		objO.CheckMandatory();
        objR.setJSJobUtilites(this);
        objR.Execute();
        
        if (isJobchain()) {
            if (objR.exist){
                spooler_log.debug3("EventExistResult=true");
                spooler_task.order().params().set_var("event_exist_result", "true");
            }else {
                spooler_log.debug3("EventExistResult=false");
                spooler_task.order().params().set_var("event_exist_result", "false");
            }
        }

        success = (
                objR.exist && objO.handle_existing_as.Value().equals("success") || 
                !objR.exist && objO.handle_not_existing_as.Value().equals("success") || 
                objR.exist && objO.handle_not_existing_as.Value().equals("error") || 
                !objR.exist && objO.handle_existing_as.Value().equals("error") || 
                objR.exist && !objO.handle_existing_as.isDirty() 
                );
        if (success) {
            logger.info("....Success:True");
        }else {
            logger.info("....Success:False");
        }
      
            
         
	} // doProcessing

}

