/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.model.objects;

import java.util.Iterator;
import java.util.List;
import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.joda.time.Interval;

import com.sos.scheduler.model.ISOSJsObjStartTimes;
import com.sos.scheduler.model.SchedulerObjectFactory;
import com.sos.scheduler.model.tools.JodaTools;
import com.sos.scheduler.model.tools.RunTimeElement;
import com.sos.scheduler.model.tools.RunTimeElements;

/**
* \class JSObjWeekdaysDay 
* 
* \brief JSObjWeekdaysDay - 
* 
* \details
*
* \section JSObjWeekdaysDay.java_intro_sec Introduction
*
* \section JSObjWeekdaysDay.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id: JSObjUltimosDay.java 20718 2013-07-18 18:16:10Z kb $
* \see reference
*
* Created on 09.02.2011 15:24:39
 */

/**
 * @author oh
 *
 */
	public class JSObjUltimosDay extends Ultimos.Day implements ISOSJsObjStartTimes {
		
		@SuppressWarnings("unused")
		private static final Logger	logger			= Logger.getLogger(JSObjUltimosDay.class);

		public JSObjUltimosDay (SchedulerObjectFactory schedulerObjectFactory) {
			super();
			objFactory = schedulerObjectFactory;
		}

		@Override
		public RunTimeElements getRunTimeElements(Interval timeRange) {
			RunTimeElements result = new RunTimeElements(timeRange);
			Iterator<Integer> it = getDay().iterator();
			while(it.hasNext()) {
				int day = it.next();
				DateTime date = JodaTools.getDayInIntervalOrNull(timeRange, (day * -1) );
				while (date != null) {
					Iterator<Period> itP = getPeriod().iterator();
					while (itP.hasNext()) {
						Period p = itP.next();
						JSObjPeriod period = new JSObjPeriod(objFactory);
						period.setObjectFieldsFrom(p);
						DateTime start = period.getDtSingleStartOrNull(date);
						if (start != null && timeRange.contains(start))
							result.add( new RunTimeElement(start,period.getWhenHoliday()));
					}
					DateTime start = JodaTools.getStartOfMonth(date.plusMonths(1));
					if (!timeRange.contains(start))
						break;
					Interval i = new Interval(start,timeRange.getEnd());
					date = JodaTools.getDayInIntervalOrNull(i, day);
				}
			}
//			Collections.sort(result, DateTimeComparator.getInstance());
			return result;
		}

		/**
		 * The default behaviour of the JobScheduler Object Model is to provide an empty List of Periods. In some 
		 * cases we want to have exactly one default period.
		 * 
		 * (non-Javadoc)
		 * @see com.sos.scheduler.model.objects.Weekdays.Day#getPeriod()
		 */
		@Override
	    public List<Period> getPeriod() {
			List<Period> list = super.getPeriod();
			WhenHoliday h = (list != null && list.size() > 0) ? list.get(0).getWhenHoliday() : WhenHoliday.SUPPRESS;
	        return (objFactory.useDefaultPeriod()) ? JSObjRunTime.getDefaultPeriod(objFactory,h) : list;
	    }

	}
