/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.testframework.h2;

/**
 * <h1>Type of database connection.</h1>
 *
 * <p>In general two connection types are supported:</p>
 * <ul>
 *      <li>
 *          <p>IN_MEMORY (correspondent with H2InMemoryConnection)</p>
 *          <p>
 *              It has no contents in the file system. The complete database is stored in memory of the running JVM only.
 *          </p>
 *      </li>
 *      <li>
 *          <p>FILE_BASED (correspondent with H2PersistentConnection or H2TemporaryConnection)</p>
 *          <p>
 *              The contents of the database are stored in the file system either in a temporary folder (H2TemporaryConnection) or in a persistent folder (H2PersistentConnection) of the file system.
 *          </p>
 *      </li>
 * </ul>
 *
 * @author Stefan Schädlich
 */
public enum H2ConnectionType {
    FILE_BASED("jdbc:h2:"),
    IN_MEMORY("jdbc:h2:mem:");

    private final String prefix;

    private H2ConnectionType(String prefix) {
        this.prefix = prefix;
    }

    public String getPrefix() {
        return prefix;
    }

}
