/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.tools.logback.db;

import com.sos.hibernate.layer.SOSHibernateDBLayer;
import org.hibernate.Query;

import java.io.File;
import java.util.List;

public class LoggingEventExceptionDBLayer extends SOSHibernateDBLayer {

    private static final String EVENT_ID = "eventId";
    private static final String I = "i";
    private static final String TRACE_LINE = "traceLine";

    private LoggingEventExceptionFilter filter = null;

    public LoggingEventExceptionDBLayer(File configurationFile) {
        super();
        this.filter = new LoggingEventExceptionFilter();
        this.setConfigurationFile(configurationFile);
        this.filter.setOrderCriteria(EVENT_ID);
        initSession();
    }

    private Query setQueryParams(String hql) {
        Query query = session.createQuery(hql);

        if (filter.getEventId() != null) {
            query.setLong(EVENT_ID, filter.getEventId());
        }

        if (filter.getI() != null) {
            query.setParameter(I, filter.getI());
        }

        if (filter.getTraceLine() != null) {
            query.setParameter(TRACE_LINE, filter.getTraceLine());
        }

        return query;
    }

    private String getWhere() {
        String where = "";
        String and = "";

        if (filter.getEventId() != null) {
            where += and + " eventId = :eventId";
            and = " and ";
        }

        if (filter.getI() != null) {
            where += and + " i = :i";
            and = " and ";
        }

        if (filter.getTraceLine() != null) {
            where += and + " traceLine = :traceLine";
            and = " and ";
        }

        return (where.isEmpty()) ? where : "where " + where;
    }

    public int deleteAll() {
        String hql = "delete from LoggingEventExceptionDBItem";
        Query query = setQueryParams(hql);
        int row = query.executeUpdate();
        return row;
    }

    public List<LoggingEventDBItem> getAll() {
        Query query = setQueryParams("from LoggingEventExceptionDBItem " + this.filter.getOrderCriteria() + this.filter.getSortMode());
        return query.list();
    }

}
