package com.sos.scheduler.engine.tests.xmlcommand.job_why;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.sos.scheduler.engine.test.TestSchedulerController;
import com.sos.scheduler.engine.test.configuration.TestConfiguration;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

import java.util.Map;

import static com.google.common.collect.Sets.difference;
import static com.sos.scheduler.engine.common.xml.XmlUtils.booleanXPath;
import static com.sos.scheduler.engine.common.xml.XmlUtils.elementXPath;
import static com.sos.scheduler.engine.common.xml.XmlUtils.loadXml;
import static com.sos.scheduler.engine.common.xml.XmlUtils.toXml;
import static com.sos.scheduler.engine.tests.xmlcommand.job_why.Configuration.jobNames;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;

public final class JobWhyIT {
    private static final Logger logger = LoggerFactory.getLogger(JobWhyIT.class);

    private static final TestSchedulerController controller = TestSchedulerController.apply(TestConfiguration.of(JobWhyIT.class));
    private static Map<String,Element> results = null;

    @BeforeClass public static void beforeClass() {
        controller.activateScheduler();
        results = executeJobWhy();
    }

    @AfterClass public static void afterClass() {
        controller.close();
        results = null;
    }

    private static Map<String,Element> executeJobWhy() {
        ImmutableMap.Builder<String,Element> b = new ImmutableMap.Builder<String,Element>();
        for (String j: jobNames)  b.put(j, executeJobWhy(j));
        return b.build();
    }
    
    private static Element executeJobWhy(String jobPath) {
        String xml = controller.scheduler().executeXml("<job.why job='" + jobPath + "'/>");
        Element result = elementXPath(loadXml(xml), "/spooler/answer/job.why");
        logger.debug(jobPath + ": " + toXml(result));
        return result;
    }

    @Test public void testJobStopped() {
        checkXPathForJobsExactly("obstacle[@state='stopped']", "c");
    }

    @Test public void testJobMinTasks() {
        checkXPathForJobsExactly("start_reason.why[@min_tasks='1']/obstacle[@in_period='false']", "minTasks");
    }

    @Test public void testJobMaxTasks() {
        checkXPathForJobsExactly("obstacle[@max_tasks='0']", "maxTasks");
    }

    @Test public void testJobchain() {
        for (Map.Entry<String,Element> i: results.entrySet())
            elementXPath(i.getValue(), jobchainXPath(i.getKey()));
    }

    @Test public void testJobchainStopped() {
        checkXPathForAllJobs("job_chain_nodes.why/job_chain_node.why/job_chain.why/obstacle[@state='stopped']");
    }

    private static void checkXPathForAllJobs(String xPath) {
        checkXPathForJobsExactly(xPath, jobNames.toArray(new String[jobNames.size()]));
    }
    
    private static void checkXPathForJobsExactly(String xPath, String... selectedJobNames) {
        for (String jobName: selectedJobNames)
            elementXPath(results.get(jobName), xPath);
        for (String jobName: difference(ImmutableSet.copyOf(selectedJobNames), ImmutableSet.copyOf(jobNames)))
            assertThat(booleanXPath(results.get(jobName), "not(obstacle[@state='stopped'])"), equalTo(false));
    }

    private static String jobchainXPath(String jobName) {
        return jobchainNodeXPath(jobName) + "/job_chain.why[@path='/j']";
    }

    private static String jobchainNodeXPath(String jobName) {
        return "job_chain_nodes.why/job_chain_node.why[@order_state='state-" + jobName + "']";
    }
}
